unit PosSaver;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TPosSaverHKEY = (_CLASSES_ROOT, _CURRENT_USER, _LOCAL_MACHINE, _USERS,
    _PERFORMANCE_DATA, _CURRENT_CONFIG, _DYN_DATA);

  TFormPosSaver = class(TComponent)
  private
    { Private declarations }
    FIniName: string;
    FIniSection: string;
    FUseRegistry: boolean;
    FRegKey: string;
    FRegRootKey: HKEY;
    F_RegRootKey: TPosSaverHKEY;

    FShowEvent  : TNotifyEvent;
    FCloseEvent : TCloseEvent;
  protected
    { Protected declarations }
    procedure SetRegRootKey(Value: TPosSaverHKEY);

    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  public
    { Public declarations }
    procedure Restore;
    procedure Save;
    constructor Create(AOwner : TComponent); override;
  published
    { Published declarations }
    property IniName: string read FIniName write FIniName;
    property IniSection: string read FIniSection write FIniSection;
    property UseRegistry: boolean read FUseRegistry write FUseRegistry;
    property RegKey: string read FRegKey write FRegKey;
    property RegRootKey: TPosSaverHKEY read F_RegRootKey write SetRegRootKey;
  end;

procedure Register;

implementation

Uses
   IniFiles, Registry;

procedure Register;
begin
  RegisterComponents('Additional', [TFormPosSaver]);
end;

constructor TFormPosSaver.Create(AOwner : TComponent);

begin
  inherited Create(AOwner);
  if (Owner is TForm) and not (csDesigning in ComponentState) then
  begin
    FShowEvent:=(Owner as TForm).OnShow;
    FCloseEvent:=(Owner as TForm).OnClose;
    (Owner as TForm).OnShow:=FormShow;
    (Owner as TForm).OnClose:=FormClose;
  end;
end;

procedure TFormPosSaver.SetRegRootKey(Value: TPosSaverHKEY);
begin
  F_RegRootKey:=Value;
  case Value of
    _CLASSES_ROOT: FRegRootKey:=HKEY_CLASSES_ROOT;
    _CURRENT_USER: FRegRootKey:=HKEY_CURRENT_USER;
    _LOCAL_MACHINE: FRegRootKey:=HKEY_LOCAL_MACHINE;
    _USERS: FRegRootKey:=HKEY_USERS;
    _PERFORMANCE_DATA: FRegRootKey:=HKEY_PERFORMANCE_DATA;
    _CURRENT_CONFIG: FRegRootKey:=HKEY_CURRENT_CONFIG;
    _DYN_DATA: FRegRootKey:=HKEY_DYN_DATA;
  end;
end;

Procedure TFormPosSaver.Restore;
Var
   Ini:TIniFile;
   r: TRegistry;
   s: string;
   SL : TStringList;
   F  : TForm;
Begin
  if FUseRegistry then
    begin
      r:=TRegistry.Create;
      r.RootKey:=FRegRootKey;
      r.OpenKey(FRegKey,false);
      try (Owner as TForm).Top:=r.ReadInteger('Top');
      except on ERegistryException do ;  end;
      try (Owner as TForm).Left:=r.ReadInteger('Left');
      except on ERegistryException do ;  end;
      try (Owner as TForm).Width:=r.ReadInteger('Width');
      except on ERegistryException do ;  end;
      try (Owner as TForm).Height:=r.ReadInteger('Height');
      except on ERegistryException do ;  end;
      r.CloseKey;
      r.Free;
    end
  else
    begin
      F:=(Owner as TForm);
      if FIniName='' then
        FIniName:=Copy (Application.ExeName, 1, Pos('.', Application.ExeName) -1) + '.ini';
      Ini:=TIniFile.Create(FIniName);
      if FIniSection='' then s:=F.Name else s:=FIniSection;
      SL:=TStringList.Create;
      Ini.ReadSectionValues(s,SL);
      if SL.Count>0 then
      begin
        F.Top:=Ini.ReadInteger(s,'Top',F.Top);
        F.Left:=Ini.ReadInteger(s,'Left',F.Left);
        F.Width:=Ini.ReadInteger(s,'Width',F.Width);
        F.Height:=Ini.ReadInteger(s,'Height',F.Height);
        try
          if F.Position=poScreenCenter then F.Position:=poDesigned;
        except
        end;
      end;
      SL.Free;
      Ini.Destroy;
    end;
End;

Procedure TFormPosSaver.Save;
Var
   Ini:TiniFile;
   r: TRegistry;
   s: string;
Begin
  if FUseRegistry then
    begin
      r:=TRegistry.Create;
      r.RootKey:=FRegRootKey;
      r.CreateKey(FRegKey);
      r.OpenKey(FRegKey,false);
      try r.WriteInteger('Top',(Owner as TForm).Top);
      except on ERegistryException do ;  end;
      try r.WriteInteger('Left',(Owner as TForm).Left);
      except on ERegistryException do ;  end;
      try r.WriteInteger('Width',(Owner as TForm).Width);
      except on ERegistryException do ;  end;
      try r.WriteInteger('Height',(Owner as TForm).Height);
      except on ERegistryException do ;  end;
      r.CloseKey;
      r.Free;
    end
  else
    begin
      if FIniName='' then
        FIniName:=Copy (Application.ExeName, 1, Pos('.', Application.ExeName) -1) + '.ini';
      Ini:=TIniFile.Create(FIniName);
      if FIniSection='' then s:=(Owner as TForm).Name else s:=FIniSection;
      Ini.WriteInteger(s,'Top',(Owner as TForm).Top);
      Ini.WriteInteger(s,'Left',(Owner as TForm).Left);
      Ini.WriteInteger(s,'Width',(Owner as TForm).Width);
      Ini.WriteInteger(s,'Height',(Owner as TForm).Height);
      Ini.Destroy;
    end;
End;


{ these function replace form's function FormShow }

procedure TFormPosSaver.FormShow(Sender: TObject);

begin
  Restore;
  try
    FShowEvent(Sender);
  except
  end;
end;

{ these function replace form's function FormClose}

procedure TFormPosSaver.FormClose(Sender: TObject; var Action: TCloseAction);

begin
  Save;
  try
    FCloseEvent(Sender,Action);
  except
  end;
end;

end.
